@@ -22,6 +22,7 @@
 
 #include "verilated_random.h"
 
+#include <iomanip>
 #include <iostream>
 #include <sstream>
 #include <streambuf>
@@ -281,37 +282,6 @@ std::string parseNestedSelect(const std::string& nested_select_expr,
     indices.push_back(idx);
     return name;
 }
-
-std::string flattenIndices(const std::vector<std::string>& indices, const VlRandomVar* const var) {
-    int flattenedIndex = 0;
-    int multiplier = 1;
-    for (int i = indices.size() - 1; i >= 0; --i) {
-        int indexValue = 0;
-        std::string trimmedIndex = indices[i];
-
-        trimmedIndex.erase(0, trimmedIndex.find_first_not_of(" \t"));
-        trimmedIndex.erase(trimmedIndex.find_last_not_of(" \t") + 1);
-
-        if (trimmedIndex.find("#x") == 0) {
-            indexValue = std::strtoul(trimmedIndex.substr(2).c_str(), nullptr, 16);
-        } else if (trimmedIndex.find("#b") == 0) {
-            indexValue = std::strtoul(trimmedIndex.substr(2).c_str(), nullptr, 2);
-        } else {
-            indexValue = std::strtoul(trimmedIndex.c_str(), nullptr, 10);
-        }
-        const int length = var->getLength(i);
-        if (length == -1) {
-            VL_WARN_MT(__FILE__, __LINE__, "randomize",
-                       "Internal: Wrong Call: Only RandomArray can call getLength()");
-            break;
-        }
-        flattenedIndex += indexValue * multiplier;
-        multiplier *= length;
-    }
-    std::string hexString = std::to_string(flattenedIndex);
-    while (hexString.size() < 8) { hexString.insert(0, "0"); }
-    return "#x" + hexString;
-}
 //======================================================================
 // VlRandomizer:: Methods
 
@@ -404,7 +374,11 @@ bool VlRandomizer::next(VlRNG& rngr) {
     f << "(define-fun __Vbv ((b Bool)) (_ BitVec 1) (ite b #b1 #b0))\n";
     f << "(define-fun __Vbool ((v (_ BitVec 1))) Bool (= #b1 v))\n";
     for (const auto& var : m_vars) {
-        f << "(declare-fun " << var.second->name() << " () ";
+        if (var.second->dimension() > 0) {
+            auto arrVarsp = std::make_shared<const ArrayInfoMap>(m_arr_vars);
+            var.second->setArrayInfo(arrVarsp);
+        }
+        f << "(declare-fun " << var.first << " () ";
         var.second->emitType(f);
         f << ")\n";
     }
@@ -444,9 +418,14 @@ bool VlRandomizer::parseSolution(std::iostream& f) {
     }
 
     f << "(get-value (";
-    for (const auto& var : m_vars) var.second->emitGetValue(f);
+    for (const auto& var : m_vars) {
+        if (var.second->dimension() > 0) {
+            auto arrVarsp = std::make_shared<const ArrayInfoMap>(m_arr_vars);
+            var.second->setArrayInfo(arrVarsp);
+        }
+        var.second->emitGetValue(f);
+    }
     f << "))\n";
-
     // Quasi-parse S-expression of the form ((x #xVALUE) (y #bVALUE) (z #xVALUE))
     char c;
     f >> c;
@@ -455,7 +434,6 @@ bool VlRandomizer::parseSolution(std::iostream& f) {
                    "Internal: Unable to parse solver's response: invalid S-expression");
         return false;
     }
-
     while (true) {
         f >> c;
         if (c == ')') break;
@@ -471,7 +449,6 @@ bool VlRandomizer::parseSolution(std::iostream& f) {
         if (name == "(select") {
             const std::string selectExpr = readUntilBalanced(f);
             name = parseNestedSelect(selectExpr, indices);
-            idx = indices[0];
         }
         std::getline(f, value, ')');
         const auto it = m_vars.find(name);
@@ -480,12 +457,34 @@ bool VlRandomizer::parseSolution(std::iostream& f) {
         if (m_randmode && !varr.randModeIdxNone()) {
             if (!(m_randmode->at(varr.randModeIdx()))) continue;
         }
-        if (indices.size() > 1) {
-            const std::string flattenedIndex = flattenIndices(indices, &varr);
-            varr.set(flattenedIndex, value);
-        } else {
-            varr.set(idx, value);
+        if (!indices.empty()) {
+            std::ostringstream oss;
+            oss << varr.name();
+            for (const auto& hex_index : indices) {
+                const size_t start = hex_index.find_first_not_of(" ");
+                if (start == std::string::npos || hex_index.substr(start, 2) != "#x") {
+                    VL_FATAL_MT(__FILE__, __LINE__, "randomize",
+                                "hex_index contains invalid format");
+                    continue;
+                }
+                const long long index = std::stoll(hex_index.substr(start + 2), nullptr, 16);
+                oss << "[" << index << "]";
+            }
+            const std::string indexed_name = oss.str();
+            const auto it = std::find_if(m_arr_vars.begin(), m_arr_vars.end(),
+                                         [&indexed_name](const auto& entry) {
+                                             return entry.second->m_name == indexed_name;
+                                         });
+            if (it != m_arr_vars.end()) {
+                std::ostringstream ss;
+                ss << "#x" << std::hex << std::setw(8) << std::setfill('0') << it->second->m_index;
+                idx = ss.str();
+            } else {
+                VL_FATAL_MT(__FILE__, __LINE__, "randomize",
+                            "indexed_name not found in m_arr_vars");
+            }
         }
+        varr.set(idx, value);
     }
     return true;
 }
